@@ -19,24 +19,24 @@ namespace Depth
 		//const double pr = 1/p;
 
 		arma::vec depth(n_x);
-
-		arma::rowvec tmp = arma::zeros<arma::rowvec>(d);
-		double sum_res = 0;
-		double tmp_sum;
     size_t k,i;
-    
+
     if(threads < 1) threads = omp_get_max_threads();
-    
-    #pragma omp parallel for shared(X, n_x, n_y, p, b, a) private(k, i, sum_res, tmp, tmp_sum) num_threads(threads)
+
+    #pragma omp parallel for shared(depth, X, Y, n_x, n_y, d, p, b, a) private(k) num_threads(threads)
 		for(k = 0; k< n_x; k++)
 		{
-			sum_res = 0;
-			for(i = 0; i < n_y; i++)
+      arma::rowvec tmp = arma::zeros<arma::rowvec>(d);
+  	  double sum_res = 0;
+		  double tmp_sum;
+
+			for(size_t i = 0; i < n_y; i++)
 			{
 				tmp = X.row(k) - Y.row(i);
 				tmp = arma::abs(tmp);
 				tmp = arma::pow(tmp, p);
 				tmp_sum = arma::sum(tmp);
+				tmp_sum = std::pow(tmp_sum, 1.0 / p);
 				tmp_sum = a*tmp_sum + b;
 				sum_res = sum_res + tmp_sum;
 			}
@@ -54,8 +54,8 @@ namespace Depth
 	{
 		arma::mat cov;
 		arma::rowvec mean;
-		
-    // if threads == -2 uses paraller function to compute 
+
+    // if threads == -2 uses paraller function to compute
     // covariance matrix and mean vector
     if(threads == -2)
     {
@@ -66,11 +66,11 @@ namespace Depth
        cov = arma::cov(Y);
        mean = arma::mean(Y);
     }
-    
+
 		return(MahalanobisDepth(X,Y,cov,mean,threads));
 	}
 
-  
+
  arma::vec MahalanobisDepth(const arma::mat& X, const arma::mat& Y, const arma::mat& cov, int threads)
  {
     arma::rowvec mean;
@@ -83,7 +83,7 @@ namespace Depth
     }
     return(MahalanobisDepth(X,Y,cov,mean,threads));
  }
- 
+
  arma::vec MahalanobisDepth(const arma::mat& X, const arma::mat& Y, const arma::rowvec& mean, int threads)
  {
     arma::mat cov;
@@ -96,19 +96,19 @@ namespace Depth
     }
     return(MahalanobisDepth(X,Y,cov,mean,threads));
  }
- 
+
  arma::vec MahalanobisDepth(const arma::mat& X, const arma::mat& Y, const arma::mat& cov, const arma::rowvec& mean, int threads)
  {
     size_t n = X.n_rows;
     arma::vec depth(n);
     arma::mat covY = cov.i();
-    
+
   	arma::rowvec tmpX;
 		double dist;
     size_t i;
-    
+
     if(threads < 1) threads = omp_get_max_threads();
-    
+
     #pragma omp parallel for shared(X,n,mean, covY) private(i, tmpX, dist) num_threads(threads)
 		for(i = 0; i < n; i++)
 		{
@@ -131,7 +131,7 @@ namespace Depth
 	arma::vec ProjectionDepth(const arma::mat& X, const arma::mat& Y, size_t nproj, int threads)
 	{
     if(threads < 1) threads = omp_get_max_threads();
-    
+
 		size_t nx = X.n_rows;
 		size_t ny = Y.n_rows;
 		size_t d  = Y.n_cols;
@@ -144,9 +144,9 @@ namespace Depth
 		arma::vec tmpProj(ny);
 		arma::rowvec medians(nproj);
 		arma::rowvec mads(nproj);
-    
+
     size_t i;
-    
+
     #pragma omp parallel for shared(nproj,Y,medians,mads,directions) private(tmpProj,i) num_threads(threads)
 		for(i = 0; i < nproj; i++)
 		{
@@ -156,7 +156,7 @@ namespace Depth
 		}
 
 		arma::rowvec tmpX(nproj);
-    
+
     #pragma omp parallel for shared(X,directions,medians,mads,nx,depth) private(i, tmpX) num_threads(threads)
 		for(i = 0; i < nx; i++)
 		{
@@ -168,16 +168,16 @@ namespace Depth
 		}
 
 		depth = 1/(1+depth);
-		
+
 		return depth;
 	}
-  
-  
+
+
   // Tukey Depth - Exact algorithm
   arma::vec TukeyDepth(const arma::mat& X, const arma::mat& Y,bool exact, int threads)
   {
     if(threads < 1) threads = omp_get_max_threads();
-    
+
     size_t n = X.n_rows;
     arma::vec depth(n);
     size_t i;
@@ -188,80 +188,80 @@ namespace Depth
 		}
     return depth;
   }
-  
+
   // MBDepth
   arma::vec MBDepth(const arma::mat& X)
   {
   //Rcpp::NumericMatrix cX(rX);
   //arma::mat X(cX.begin(), cX.nrow(), cX.ncol(), false);
-  
+
     arma::vec depth(X.n_rows); depth.zeros();
-    
+
     size_t d = X.n_cols;
     for(size_t i = 0; i < d; i++)
     {
       depth += MBD::depthForCol(X.col(i));
     }
-    
+
     depth = depth / (d * Rf_choose(X.n_rows, 2));
-    
+
     return depth;
   }
-  
-  
+
+
   arma::vec MBDepth(const arma::mat& X, const arma::mat& Y)
   {
     //Rcpp::NumericMatrix cY(rY);
     //arma::mat Y(cY.begin(), cY.nrow(), cY.ncol(), false);
     //Rcpp::NumericMatrix cX(rX);
     //arma::mat X(cX.begin(), cX.nrow(), cX.ncol(), false);
-    
+
     arma::vec depth(X.n_rows); depth.zeros();
-    
+
     size_t d = X.n_cols;
     for(size_t i = 0; i < d; i++)
     {
       depth += MBD::depthFuncForRef(X.col(i), Y.col(i));
     }
-    
+
     depth = depth / (d * Rf_choose(Y.n_rows, 2));
-    
+
     return depth;
   }
-  
-  
+
+
 }
- 
- 
+
+
  /// Utils functions for MBDepth
 namespace MBD
 {
-  arma::vec depthForCol(arma::vec x) 
+  arma::vec depthForCol(arma::vec x)
   {
     arma::uvec idx = sort_index( x );
     arma::vec sorted = x(idx);
     size_t n = idx.n_elem;
     arma::uvec equal(n); equal.ones();
     arma::uvec lowidx(n);
-    
-    
-    
+
+
+
     size_t pos;
     size_t ti;
     size_t eq;
-    
-    
+
+
     for(size_t i = 0; i < n; i++)
     {
-      
+
       pos = idx(i);
       lowidx(pos) = i;
         eq = 0;
-      
-        if(i > 0) 
+
+        if(i > 0)
         {
           ti = i - 1;
-                
+
           while(sorted(i) == sorted(ti))
           {
             lowidx(pos) -= 1;
@@ -270,7 +270,7 @@ namespace MBD
             ti--;
           }
         }
-        
+
         ti = i + 1;
         while((ti < n) && (sorted(i) == sorted(ti)))
         {
@@ -278,41 +278,41 @@ namespace MBD
             ti++;
         }
         equal(pos) = equal(pos) + eq;
-        
+
     }
-    
-    
+
+
     equal = equal + lowidx;
     arma::vec depth(n);
-    
-    for(size_t i = 0; i < n; i++) 
+
+    for(size_t i = 0; i < n; i++)
     {
       double multiplicity = equal(i) - lowidx(i);
       depth(i) = lowidx(i) * (n - (equal(i))) + multiplicity * (n - equal(i) + lowidx(i));
       depth(i) += Rf_choose(multiplicity, 2);
     }
-    
+
     return depth;
   }
-  
+
   ///////////////////////////
-  
-  arma::vec depthFuncForRef(arma::vec X,arma::vec Y) 
+
+  arma::vec depthFuncForRef(arma::vec X,arma::vec Y)
   {
     size_t n = X.n_elem;
     size_t n0 = Y.n_elem;
     arma::vec depth(n);
-    
+
     Y = arma::sort(Y);
-    
+
     double index1;
     double index2;
     double tval;
     size_t j;
-    
-    for(size_t i = 0; i < n; i++) 
+
+    for(size_t i = 0; i < n; i++)
     {
-            index1 = 0; 
+            index1 = 0;
             index2 = 0;
             tval = X[i];
             j = 0;
@@ -322,14 +322,14 @@ namespace MBD
               if(tval > Y[j]) index1++;
               j++;
             }
-            
+
             double multiplicity = index2 - index1;
-            depth[i] =  (index1 + multiplicity) * (n0 - index1 - multiplicity) + 
+            depth[i] =  (index1 + multiplicity) * (n0 - index1 - multiplicity) +
             multiplicity * (index1 + (multiplicity - 1)/2);
     }
-   
+
       return depth;
   }
-  
+
 }
 
